<?php
/**
 * http://new-dev.ru/
 * author GoldSoft <newdevexpert@gmail.com>
 * Copyright (c) New-Dev.ru
 */

namespace TraffControl;

defined('DATALIFEENGINE') or exit('Access Denied');

class Helper extends ObjectAbstraction
{
	const TAB = "\t";
	const SET_AJAX_CONTENT = 1;
	const APPEND_AJAX_CONTENT = 2;

	protected $css_files = array();
	protected $css_code_array = array();
	protected $js_files = array();
	protected $js_code_array = array();
	protected $html_code_array = array();
	
	private $only_once_hashes = array();
	private $ajax_result_vars = array();
	
	public function getJsOptions($additional_options = array()) {
		$js_options = array(
			'ADMIN_URL' => $this->traffcontrol->module_admin_url,
			'SITE_URL' => $this->traffcontrol->site_url,
			'SITE_AJAX_URL' => $this->traffcontrol->site_ajax_url,
			'ONLY_ONCE_VOTE' => $this->traffcontrol->config['only_once_vote'],
			'USER_HASH' => $this->traffcontrol->user_hash,
			'IN_ADMINPANEL' => defined('IN_ADMINPANEL'),
		);
		
		$js_options += $additional_options;
		
		return json_encode($js_options);
	}
	
	public function sendAjaxContent($selector, $content, $mode = self::SET_AJAX_CONTENT) {
		$arr = array(
			'response' => $content,
		);
		
		if ($mode == self::SET_AJAX_CONTENT)
			$arr['container'] = $selector;
		else
			$arr['append_container'] = $selector;

		$this->printAjaxResult($arr);
	}
	
	public function showMsgOk($text, $title = 'Информация', $redirect = '') {
		$this->showMsgOkNotTranslate(__($text, false), __($title, false), $redirect);
	}
	
	public function showMsgOkNotTranslate($text, $title = '', $redirect = '') {
		$title or $title = lang('Информация');
		
		$arr = array(
			'status' => 'ok',
			'msg' => array(
				'title' => $title,
				'text' => $text,
			),
			'redirect' => $redirect,
		);

		$this->printAjaxResult($arr);
	}
	
	public function showMsgError($text, $title = 'Ошибка', $redirect = '') {
		$this->showMsgErrorNotTranslate(__($text, false), __($title, false), $redirect);
	}
	
	public function showMsgErrorNotTranslate($text, $title = '', $redirect = '') {
		$title or $title = lang('Ошибка');
		
		$arr = array(
			'status' => 'error',
			'msg' => array(
				'title' => $title,
				'text' => $text,
			),
			'redirect' => $redirect,
		);

		$this->printAjaxResult($arr);
	}
	
	public function printAjaxResult($data = array()) {
		$data = array_merge($data, $this->ajax_result_vars);
		
		if (!empty($data['status']) and !empty($data['msg'])) {
			if (empty($data['sticky_options'])) {
				$data['sticky_options'] = array();
			}
			
			if (!empty($data['redirect'])) {
				$data['sticky_options'] += array('autoclose' => false);
			}
			
			$data['sticky_options'] += array('autoclose' => is_numeric($this->traffcontrol->config['sticky_autoclose']) ? $this->traffcontrol->config['sticky_autoclose'] : 5000);
		}
		
		echo json_encode($data);
		exit;
	}
	
	public function assignAjaxResultVar($key, $value) {
		$this->ajax_result_vars[$key] = $value;
	}
	
	public function fixMissedElements($missed_elements, $post, $val = 0) {
		$output = $post;
		
		if ($missed_elements) {
			foreach ($missed_elements as $key => $value) {
				$str = str_replace(array('][', '[', ']'), ',', $value);
				$str = trim($str, ',');
				$keys_arr = explode(',', $str);

				$reference =& $output;
				foreach ($keys_arr as $key) {

					if (!$reference or !array_key_exists($key, $reference)) {
						$reference[$key] = array();
					}
					
					$reference = &$reference[$key];
				}
				
				$reference = $val;
				unset($reference);
			}

			$output = $this->arrayMergeRecursiveEx($output, $post);
		}

		return $output;
	}
	
	public function arrayMergeRecursiveEx(array $array1, array $array2, $exclude_keys = array()) {
		$merged = $array1;

		foreach ($array2 as $key => $value) {
			if (in_array($key, $exclude_keys)) {
				$merged[$key] = $value;
				continue;
			}
			
			if (is_array($value) and isset($merged[$key]) and is_array($merged[$key])) {
				$merged[$key] = $this->arrayMergeRecursiveEx($merged[$key], $value, $exclude_keys);
			} elseif (is_numeric($key)) {
				if (!in_array($value, $merged))
					$merged[] = $value;
			} else
				$merged[$key] = $value;
		}

		return $merged;
	}

	public function arrayDiffRecursive($array1, $array2) {
		$result = array();

		foreach ($array1 as $key => $value) {
			if (array_key_exists($key, $array2)) {
				if (is_array($value)) {
					$temp_result = $this->arrayDiffRecursive($value, $array2[$key]);
					if (count($temp_result)) {
						$result[$key] = $temp_result;
					}
				} else {
					if ($value != $array2[$key]) {
						$result[$key] = $value;
					}
				}
			} else {
				$result[$key] = $value;
			}
		}
		return $result;
	}

	public function getSiteUrl($config) {
		$protocol = $this->getSiteProtocol();

		if (!$config['http_home_url']) {
			$url = $protocol.'://'.$_SERVER['HTTP_HOST'];
		} elseif (strpos($config['http_home_url'], '//') === 0) {
			$url = $protocol.':'.$config['http_home_url'];
		} elseif (strpos($config['http_home_url'], '/') === 0) {
			$url = $protocol.'://'.$_SERVER['HTTP_HOST'].$config['http_home_url'];
		} else {
			$url = $config['http_home_url'];
			if ($protocol == 'https' and stripos($url, 'http://') !== false)
				$url = str_ireplace('http://', 'https://', $url);
		}
		$url = rtrim($url, '/').'/';
		return $url;
	}

	public function getSiteProtocol() {
		if (function_exists('isSSL'))
			return isSSL() ? 'https' : 'http';

		if ($str = safe_array_access($_SERVER, 'HTTP_X_FORWARDED_PROTO') and in_array($str, array('http', 'https')))
			return $str;

		if ($str = safe_array_access($_SERVER, 'HTTPS') and strtolower($str) == 'on')
			return 'https';

		return 'http';
	}
	
	public function writeArrayToFile($filepath, $config_array) {
		if (is_writeable($filepath)) {
			$str = var_export($config_array, true);
			$str = str_replace('  ', "\t", $str);
			$str = trim(preg_replace('#\s*array\s+\(#U', ' array(', $str));
			$str = '<?php'.PHP_EOL.'$traffcontrol_config = '.$str.';'.PHP_EOL.'?>';
			return file_put_contents($filepath, $str);
		}
	}
	
	public function showRow($title = '', $description = '', $field = '', $container_part = '', $col_md_values = array(), $is_required = false, $convert = true) {
		$col_md_values or $col_md_values = array(5, 7);
		
		if ($is_required) {
			$container_part .= ' class="tr_f_req"';
			if (preg_match('#\s+name="([^"]+)"#', $field, $matches))
				$name = $matches[1];
			else
				trigger_error('No matches', E_USER_ERROR);
			$field .= $this->printRequiredHtml($title, $name);
		}
		
		list($col_md_1, $col_md_2) = $col_md_values;
		
		if ($convert) {
			$title = __($title);
			$description = __($description);
		}
		
		echo "<tr {$container_part}>
		   <td class=\"col-xs-6 col-sm-6 col-md-{$col_md_1}\"><div class=\"media-heading text-semibold\">".$title."</div><span class=\"text-muted text-size-small hidden-xs\">".$description."</span></td>
		   <td class=\"col-xs-6 col-sm-6 col-md-{$col_md_2}\">{$field}</td>
		   </tr>";
	}
	
	public function makeCheckbox($name, $checked, $tag_part = '') {
		return '<input type="checkbox" name="'.$name.'" value="1" class="switch '.($this->isVersionLess('12') ? 'icheck' : '').'" '.($checked ? 'checked ' : ' ').$tag_part.' /><input type="hidden" name="fix_missed_integer[]" value="'.$name.'" />';
	}

	public function makeDropdown($options, $name, $selected, $tag_part = '') {
		$output = "<select class=\"uniform\" name=\"{$name}\" {$tag_part}>\r\n";
		foreach ($options as $value => $description) {
			$output .= "<option value=\"{$value}\"";
			if ($selected == $value) {
				$output .= " selected ";
			}
			$output .= ">{$description}</option>\n";
		}
		$output .= "</select>";
		return $output;
	}

	public function makeInputText($name, $value) {
		return '<input type="text" name="'.$name.'" value="'.$this->htmlspecialchars($value).'" class="form-control" />';
	}
	
	public function makeTextarea($name, $value, $tag_part = '') {
		return '<textarea class="classic width-500" name="'.$name.'" id="'.$name.'" '.$tag_part.' spellcheck="false">'.$value.'</textarea>';
	}
	
	public function makeLinkedPartArray($params) {
		$name = safe_array_access($params, 'name');
		
		if (isset($params['possible']))
			$possible = $params['possible'];
		
		if (isset($params['value'])) {
			$value = $params['value'];
		} elseif (isset($params['names'])) {
			$value = get_nested_var($this->config, $params['names']);
		}
		
		$key = isset($params['key']) ? $params['key'] : $name;
		
		return $this->makeLinkedPart($name, $possible, $value);
	}
	
	public function makeLinkedPart($name, $possible = 1, $value = null) {
		if (is_null($value))
			$value = $this->config[$name];
		
		if (is_array($possible)) {
			$exists = in_array($value, $possible);
			$possible = implode(' ', $possible);
		} else {
			$exists = $value == $possible;
		}
		
		return ' data-linked_container_'.$name.'="'.$possible.'"'.($exists ? '' : ' style="display:none;"');
	}
	
	public function printRequiredHtml($title, $name) {
		return '<input type="hidden" name="required_fields[]" value="'.$name.'" /><input type="hidden" name="required_fields_titles['.base64_encode($name).']" value="'.__($title).'" />';
	}
	
	public function emptyRequiredFields($source_array = array()) {
		if ($required_fields = safe_array_access($source_array, 'required_fields')) {
			$errors = array();
			
			foreach ($required_fields as $key => $value) {
				$str = str_replace(array('][', '[', ']'), ',', $value);
				$str = trim($str, ',');
				$keys_arr = explode(',', $str);
				
				$arr = $source_array;

				foreach ($keys_arr as $key) {
					if ($arr and is_array($arr) and !empty($arr[$key])) {
						$arr = $arr[$key];
					} else {
						$errors[] = lang('Пустое поле').' &laquo;'.safe_array_access($source_array, 'required_fields_titles', base64_encode($value)).'&raquo;';
					}
				}
			}
			
			if ($errors) {
				$this->showMsgErrorNotTranslate(implode('<br>', $errors));
			}
		}
	}
	
	public function mkdir($dir) {
		if (!file_exists($dir)) {
			if (!mkdir($dir, 0755, true)) {
				trigger_error('No permission to create dir: ['.$dir.']', E_USER_ERROR);
			}
		}
		
		return $dir;
	}

	public function rmdir($dir, $delete_self = true, $exclude = '') {
		if (is_dir($dir)) {
			$entries = scandir($dir);
			
			foreach ($entries as $entry) {
				if ($entry != '.' and $entry != '..') {
					if (is_dir($dir.'/'.$entry)) {
						$this->rmdir($dir.'/'.$entry);
					} else {
						if ($exclude and $exclude == $entry) {
							$delete_self = false;//anyway, empty folder can't delete
						} else {
							$this->unlink($dir.'/'.$entry, false);
						}
					}
				}
			}

			return $delete_self ? rmdir($dir) : true;
		}
	}

	public function unlink($path, $verify_exists = true) {
		if (!$verify_exists or is_file($path)) {
			if (!unlink($path)) {
				trigger_error('No permission to delete: ['.$path.']', E_USER_ERROR);
			}
			
			return true;
		}
		
		return false;
	}

	public function htmlspecialchars($text) {
		return htmlspecialchars($text, ENT_QUOTES, $this->dle->config['charset']);
	}

	public function getUpgradeTabCaption() {
		$cache_key = $this->getUpgradeCacheKey();
		if ($cache = $this->getCache($cache_key)) {
			$data = json_decode($cache, true);

			if ($this->isActualUpgrade($data['version'])) {
				return '<sup style="color:red">'.__('Есть обновления').'</sup>';
			} else {
				return '<sup>'.__('Нет').'</sup>';
			}
		}
		return '<span id="upgrade_tab_caption_container"><img src="'.$this->traffcontrol->site_url.'engine/skins/images/loading.gif" alt="" /></span>';
	}

	public function getUpgradeCacheKey() {
		$str = $this->traffcontrol->alias.'_check_updates_';
		return $str.$this->getCacheKey($str);
	}
	
	public function getCacheKey($id = '') {//ключ кэша
		$str = $this->traffcontrol->alias.'_';
		return $str.md5($str.$id);
	}
	
	public function getCache($cache_key, $expired_seconds = 345600) {
		$result = false;
		
		$filepath = $this->traffcontrol->cache_path.$cache_key.'.tmp';
		if (file_exists($filepath)) {
			if (time() > filemtime($filepath) + $expired_seconds) {
				unlink($filepath);
			} else {
				$result = file_get_contents($filepath);
			}
		}

		return $result;
	}

	public function putCache($cache_key, $data) {
		$filepath = $this->mkdir($this->traffcontrol->cache_path).$cache_key.'.tmp';
		return file_put_contents($filepath, $data);
	}
	
	public function loadUrl($url, $post = null, $filepath = '') {
		$ch = curl_init($url);
		curl_setopt($ch, CURLOPT_TIMEOUT, 2);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 2);
		if ($filepath) {
			$fp = fopen($filepath, 'w+');
			curl_setopt($ch, CURLOPT_FILE, $fp);
		} else {
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		}
		
		if ($post) {
			curl_setopt($ch, CURLOPT_POST, 1);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $post);
		}
		
		@curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_AUTOREFERER, true);
		curl_setopt($ch, CURLOPT_REFERER, $this->traffcontrol->site_url);
		$response = curl_exec($ch);
		curl_close($ch);
		if ($filepath)
			fclose($fp);

		return $response;
	}

	public function isActualUpgrade($new_version) {
		return ($new_version and version_compare($new_version, $this->traffcontrol->config['ignored_update_version'], '>') and version_compare($new_version, $this->traffcontrol->config['version'], '>'));
	}

	public function installUpgrade($temp_filepath, $version) {
		require_once $this->traffcontrol->module_path.'admin/lib/pclzip/pclzip.lib.php';
		require_once $this->traffcontrol->module_path.'admin/lib/sql_parse.php';
		require_once $this->traffcontrol->module_path.'admin/lib/xcopy/xcopy.php';

		if (file_exists($temp_filepath) and filesize($temp_filepath) > 100) {
			$dir = str_replace('.zip', '', $temp_filepath);

			if ($this->mkdir($dir)) {
				$archive = new \PclZip($temp_filepath);

				if ($archive->extract(PCLZIP_OPT_PATH, $dir) != 0) {
					copy_folder($dir.'/files', ROOT_DIR, false, false);

					$str = $dir.'/upgrade.php';
					if (file_exists($str))
						include $str;

					$this->rmdir($dir);

					return true;
				} else {
					$this->showMsgErrorNotTranslate('Error : '.$archive->errorInfo(true));
				}
			}
		}
	}

	public function addCSSFile($url, $footer = false, $minify_off = false, $media = null, $attribs = array()) {
		$key = $footer ? 'footer' : 'header';
		
		$this->css_files[$url]['footer'] = $footer;
		$this->css_files[$url]['media'] = $media;
		$this->css_files[$url]['attribs'] = $attribs;

		return $this;
	}
	
	public function addCSSCode($content, $footer = false) {
		$key = $footer ? 'footer' : 'header';
		
		$this->css_code_array[$key][] = array(
			'content' => $content,
		);

		return $this;
	}
	
	public function addJsFile($url, $footer = false, $minify_off = false, $defer = false, $async = false) {
		$key = $footer ? 'footer' : 'header';
		
		$this->js_files[$url]['footer'] = $footer;
		$this->js_files[$url]['defer'] = $defer;
		$this->js_files[$url]['async'] = $async;

		return $this;
	}
	
	public function addJsCode($content, $footer = false, $on_ready = false, $only_once_check = false) {
		if ($only_once_check) {
			$hash = md5($content);
			if (in_array($hash, $this->only_once_hashes)) {
				return $this;
			} else {
				$this->only_once_hashes[] = $hash;
			}
		}
		
		$key = $footer ? 'footer' : 'header';
		
		$this->js_code_array[$key][] = array(
			'content' => $content,
			'on_ready' => $on_ready,
		);
		
		return $this;
	}
	
	public function addHtmlCode($content, $footer = false) {
		$key = $footer ? 'footer' : 'header';

		if (empty($this->html_code_array[$key]))
			$this->html_code_array[$key] = $content;
		else
			$this->html_code_array[$key] .= PHP_EOL.$content;
		
		return $this;
	}
	
	public function renderCSSLinks($footer = false) {//Generate stylesheet links
		$html = '';
		$tab = self::TAB;
		
		$css_files = array_filter($this->css_files, function ($file) use ($footer) {
			return $footer ? $file['footer'] : !$file['footer'];
		});

		if ($css_files) {
			if ($css_files) {
				foreach ($css_files as $url => $params) {
				
					$html .= $tab.'<link rel="stylesheet" href="'.$url.'" type="text/css"';

					if (!is_null($params['media'])) {
						$html .= ' media="'.$params['media'].'"';
					}

					$html .= $this->buildTagAttributes($params['attribs']);
					$html .= ' />'.PHP_EOL;
				}
			}
		}
		
		return $html;
	}
	
	public function renderCSSCode($footer = false) {//Generate stylesheet declarations
		$html = '';
		$tab = self::TAB;
		$key = $footer ? 'footer' : 'header';
		
		if (isset($this->css_code_array[$key])) {
			$html .= $tab.'<style type="text/css">'.PHP_EOL;
			
			foreach ($this->css_code_array[$key] as $value) {
			
				$content = $value['content'];
				
				if (strpos($content, "\n") !== false) {
					$content = str_replace("\n", "\n".$tab, $content);
					$html .= $tab.$tab.$content.PHP_EOL;
				} else {
					$html .= $tab.$content.PHP_EOL;
				}
			}

			$html .= $tab.'</style>'.PHP_EOL;
		}
	
		return $html;
	}
	
	public function renderJsLinks($footer = false) {//Generate script file links
		$html = '';
		$tab = self::TAB;
		
		$js_files = array_filter($this->js_files, function ($file) use ($footer) {
			return $footer ? $file['footer'] : !$file['footer'];
		});
		
		if ($js_files) {
			foreach ($js_files as $url => $params) {
			
				$html .= $tab.'<script src="'.$url.'" type="text/javascript"';

				if ($params['defer']) {
					$html .= ' defer="defer"';
				}

				if ($params['async']) {
					$html .= ' async="async"';
				}
				
				if ($this->isWin1251Charset()) {
					$html .= ' charset="utf-8"';
				}
				
				$html .= '></script>'.PHP_EOL;
			}
		}
		
		return $html;
	}
	
	public function renderJsCode($footer = false) {//Generate script declarations
		$html = '';
		$tab = self::TAB;
		$key = $footer ? 'footer' : 'header';
		
		if (isset($this->js_code_array[$key])) {
			$temp = '';
			$html .= $tab.'<script type="text/javascript">'.PHP_EOL;

			//немедленное выполнение JS кода
			foreach ($this->js_code_array[$key] as $value) {
			
				$content = $value['content'];
				
				if ($value['on_ready']) {
					$content = str_replace("\n", "\n".$tab.$tab, $content);
					$temp .= $tab.$tab.$content.PHP_EOL;
				} else {
					if (strpos($content, "\n") !== false) {
						$content = str_replace("\n", "\n".$tab, $content);
						$html .= $tab.$tab.$content.PHP_EOL;
					} else {
						$html .= $tab.$content.PHP_EOL;
					}
				}
			}
			
			//выполнение JS кода после загрузки страницы
			if ($temp) {
				$html .= PHP_EOL;
				$html .= $tab.'jQuery(document).ready(function($) {'.PHP_EOL;
				$html .= $temp;
				$html .= $tab.'});'.PHP_EOL;
			}

			$html .= $tab.'</script>'.PHP_EOL;
		}
	
		return $html;
	}
	
	public function renderHtmlCode($footer = false) {//Generate HTML declarations
		$tab = self::TAB;
		$key = $footer ? 'footer' : 'header';
		
		if (isset($this->html_code_array[$key])) {
			$content = $this->html_code_array[$key];
			$content = str_replace("\n", "\n".$tab, $content);
			return $tab.$content.PHP_EOL;
		}
	}

	public function buildTagAttributes(array $array) {
		$output = '';
		
		foreach ($array as $key => $value) {
			$output .= ' '.$key.'="'.$value.'"';
		}
		
		return $output;
	}
	
	public function hint($hint_name, $hint_content) {
		$id = totranslit(basename($hint_name), true, false);
		
		if (empty($_COOKIE[$this->traffcontrol->alias.'_hide_hint_'.$id])) {
			$numargs = func_num_args();
			if ($numargs == 2) {
				$hint_content = __($hint_content);
			} else {
				$arg_list = func_get_args();
				$arg_list = array_slice($arg_list, 2);
				$hint_content = vsprintf(__($hint_content), $arg_list);
			}
			
			return '
			<div class="alert alert-info">
				<a href="#" data-id="'.$id.'" class="close hasTip" data-dismiss="alert" title="'.__('Закрыть и больше не показывать эту подсказку.').'">&times;</a>
				'.nl2br($hint_content).'
			</div>';
		}
	}
	
	public function getHelpInfo($text) {
		return '<i class="help-button visible-lg-inline-block text-primary-600 fa fa-question-circle position-right" data-rel="popover" data-trigger="hover" data-placement="right" data-content="'.__($text).'"></i>';
	}
	
	public function lang($text) {
		if ($text and $language = $this->traffcontrol->config['language']) {
			$filepath = $this->traffcontrol->module_path.'language/'.$language.'/language.php';
			
			require $filepath;
			
			if (isset($traffcontrol_config[$text])) {
				return $traffcontrol_config[$text];
			} else {
				$traffcontrol_config[$text] = '';
				$this->writeArrayToFile($filepath, $traffcontrol_config);
				return '';
			}
		}
	}
	
	public function createLanguageFile($config_fields) {
		if ($language = safe_array_access($config_fields, 'language') and $language != $this->traffcontrol->config['language']) {
			$language = preg_replace('#\W#', '', $language);
			$filepath = $this->mkdir($this->traffcontrol->module_path.'language/'.$language).'/language.php';
			if (!file_exists($filepath)) {
				$result = sprintf(lang('. Создан новый языковой файл: <b>%s</b>. После посещения всех страниц нуждающихся в переводе, файл автоматически заполняется строками, которые необходимо перевести'), str_replace(ROOT_DIR, '', $filepath));
				file_put_contents($filepath, '');
				$this->writeArrayToFile($filepath, array());
				return $result;
			} else {
				return sprintf(lang('. Подключен языковой файл: <b>%s</b>.'), str_replace(ROOT_DIR, '', $filepath));
			}
		}
	}
	
	public function isWin1251Charset() {
		return $this->dle->config['charset'] != 'utf-8';
	}
	
	public function isVersionLess($version) {//true - если версии меньше указанной
		return version_compare($this->dle->config['version_id'], $version, '<');
	}
	
	public function isVersionLarger($version) {//true - если версии больше указанной
		return version_compare($this->dle->config['version_id'], $version, '>');
	}
	
	public function truncateHtml($text, $maxchar = 70, $ellipsis = true) {
		$patterns = array(
			'#<!--dle_spoiler(.+)<!--spoiler_text-->#Usi',
			'#<!--spoiler_text_end-->(.+)<!--/dle_spoiler-->#Usi',
			'#\[attachment=(.*)\]#Usi',
			'#\[hide(.*)\](.+)\[/hide\]#Usi',
		);
		$text = preg_replace($patterns, '', $text);

		$text = str_replace('><', '> <', $text);
		$text = strip_tags($text, '<br>');

		$search = array('<br />', '<br>', "\n", "\r");
		$replace = array(' ', ' ', ' ', '');
		$text = trim(str_replace($search, $replace, $text));
		$text = preg_replace('#\s+#', ' ', $text);

		$cutted = false;

		if ($maxchar) {
			$char = mb_strlen($text, 'UTF-8');

			if ($char > $maxchar) {
				$text = mb_substr($text, 0, $maxchar, 'UTF-8');
				$i = mb_strlen(strrchr($text, ' '), 'UTF-8');
				$text = mb_substr($text, 0, mb_strlen($text) - $i, 'UTF-8');
				$cutted = true;
			}
		}

		$text = trim($text, ',-');

		if ($cutted and $ellipsis)
			$text .= '..';

		return $text;
	}	
}