<?php
/**
 * http://new-dev.ru/
 * author GoldSoft <newdevexpert@gmail.com>
 * Copyright (c) New-Dev.ru
 */

namespace TraffControl;

defined('DATALIFEENGINE') or exit('Access Denied');

class TraffControl
{
	const CFG_FILEPATH = '/data/config.traffcontrol.php';
	const UPDATES_API_URL = 'https://new-dev.ru/updates_api.php?product_id=62&action=checknewmodule';
	
	const ADMIN_ID = 1;
	const ADMIN_USERGROUP_ID = 1;
	const VISITOR_USERGROUP_ID = 4;
	const GUEST_USERGROUP_ID = 5;
	
	const USER_ACCESS_ACTION_STUB = 1;
	const USER_ACCESS_ACTION_USER_DOMAIN = 2;
	
	const SEARCHENGINE_ACCESS_ACTION_NONE = 0;
	const SEARCHENGINE_ACCESS_ACTION_BASE_DOMAIN = 1;
	const SEARCHENGINE_ACCESS_ACTION_BLANK_PAGE = 2;
	const SEARCHENGINE_ACCESS_ACTION_NOINDEX = 3;
	
	const DEBUG = 0;
	
	protected $config = array();
	protected $site_url = '';
	protected $module_url = '';
	protected $rel_module_url = '';
	protected $module_path = '';
	protected $admin_url = '';
	protected $module_admin_url = '';
	protected $alias = 'traffcontrol';
	protected $cache_path = '';
	protected $tpl = null;
	protected $disable_index = false;
	
	private $current_date = null;
	private static $instance;
	
    public static function instance() {
        if (!isset(self::$instance)) {
            self::$instance = new self();
        }
        return self::$instance;
    }
	
	public function __construct() {

		$this->assignConfig();
		
		require ENGINE_DIR.'/modules/traffcontrol/classes/objectabstraction.class.php';
		require ENGINE_DIR.'/modules/traffcontrol/classes/dlevars.class.php';
		$this->dle = new DleVars;
		
		require ENGINE_DIR.'/modules/traffcontrol/classes/helper.class.php';
		$this->helper = new Helper;
		
		$this->site_url = $this->helper->getSiteUrl($this->dle->config);
		$this->site_url_ws = rtrim($this->site_url, '/');
		$this->rel_module_url = '/engine/modules/traffcontrol/';
		$this->module_url = $this->site_url_ws.$this->rel_module_url;
		$this->module_path = ENGINE_DIR.'/modules/traffcontrol/';
		$this->admin_url = $this->site_url.$this->dle->config['admin_path'];
		$this->module_admin_url = $this->admin_url.'?mod=traffcontrol';
		$this->site_ajax_url = $this->module_url.'site/ajax.php';
		
		$this->cache_path = ENGINE_DIR.'/cache/traffcontrol/';
		$this->upload_url = $this->site_url.'uploads/traffcontrol/';
		$this->upload_path = ROOT_DIR.'/uploads/traffcontrol/';
		
		require $this->module_path.'classes/db.class.php';
		$this->db = new DB;
	}
	
	public function __get($property) {
		if (isset($this->{$property}) or property_exists($this, $property)) {
			return $this->{$property};
		}
		
		if (in_array($property, array('user_ip', 'long_user_ip'))) {
			$this->user_ip = get_ip();
			$this->long_user_ip = ip2long($this->user_ip);
			return $this->{$property};
		}

		if (in_array($property, array('user_id', 'user_group', 'user_login')) and $this->dle->member_id) {
			$this->user_id = (int)safe_array_access($this->dle->member_id, 'user_id');
			$this->user_group = (int)safe_array_access($this->dle->member_id, 'user_group');
			$this->user_login = safe_array_access($this->dle->member_id, 'name');
			return $this->{$property};
		}
		
		if (in_array($property, array('user_hash'))) {
			if (!$this->user_hash = $this->dle->dle_login_hash) {
				$this->user_hash = $this->getOldVersionUserHash();
			}
			return $this->{$property};
		}
	}
	
	public static function checkPlugins($filepath) {
		if (class_exists('\DLEPlugins', false))
			return \DLEPlugins::Check($filepath);
		
		return $filepath;
	}
	
	public function exec() {
		$is_bot = $this->isBot();
		
		if ($this->isBasedomain()) {
			$open_site = false;
			$name = $this->config['cookie_that_open_access_name'];
			
			if ($this->config['cookie_that_open_access_on'] and !empty($_COOKIE[$name])) {
				$open_site = true;
			} elseif ($is_bot) {
				$open_site = true;
			} elseif ($ref = safe_array_access($_SERVER, 'HTTP_REFERER')) {
				$se = array('google.', 'yandex.', 'ya.', 'rambler.', 'yahoo.', 'bing.', 'mail.');
				
				foreach ($se as $value) {
					if (strpos($ref, $value) !== false) {
						$open_site = true;
						break;
					}
				}
				
				if (!$open_site) {
					$basedomain = parse_url($this->getBaseUrl(), PHP_URL_HOST);
					
					if (strpos($ref, '//'.$basedomain.'/') !== false) {
						$open_site = true;
					}
				}
				
				if ($open_site and $this->config['user_domain_only_for_mobile']) {
					require $this->module_path.'site/lib/Mobile_Detect/Mobile_Detect.php';
					
					$detect = new \Mobile_Detect\Mobile_Detect;
					
					if (!$detect->isMobile()) {
						$open_site = false;
					}
				}
			}
			
			
			if ($open_site) {
				if (!$is_bot) {
					$this->redirectUserToUserdomain();
				}
			} else {
				$this->eventUserDirectAccessAction();
			}
			
		} else {
			if ($is_bot) {
				$this->eventSearchengineAccessAction();
			}
			
			if ($this->config['user_domain_noindex']) {
				$this->disable_index = true;
			}
		}
	}
	
	public function isBasedomain() {
		$basedomain = parse_url($this->getBaseUrl(), PHP_URL_HOST);
		
		return $this->getHeaderHost() == $basedomain;
	}
	
	public function getHeaderHost() {
		if ($str = safe_array_access($_SERVER, 'HTTP_X_HOST')) {
			return $str;
		}

		if ($str = safe_array_access($_SERVER, 'HTTP_HOST')) {
			return $str;
		}

		return null;
	}
	
	public function getCurrentSiteUrl() {
		return $this->helper->getSiteProtocol().'://'.$this->getHeaderHost().'/';
	}
	
	public function isBot() {
		if ($this->config['bot_emulation_ip_on']) {
			if ($this->config['bot_emulation_ip'] == $this->user_ip) {
				return true;
			}
		}
		
		if ($user_agent = safe_array_access($_SERVER, 'HTTP_USER_AGENT')) {
			
			$botlist = explode("\n", $this->config['bot_list']);
			$botlist = array_map('trim', $botlist);
			$botlist = array_filter($botlist, 'strlen');
			
			if (!$botlist)
				trigger_error('Error in line: '.__LINE__, E_USER_ERROR);
			
			foreach ($botlist as $bot) {
				if (strpos($user_agent, $bot) !== false) {
					return true;
				}
			}
		} else {
			return true;
		}
		
		return false;
	}
	
	public function eventUserDirectAccessAction() {
		if ($this->config['user_direct_access_action'] == self::USER_ACCESS_ACTION_STUB) {
			if ($this->existsOpenSiteReference() or $this->existsOpenSiteGetParams()) {
				$this->redirectUserToUserdomain();
			} else {
				switch ($this->config['http_response_code']) {
					case '500': header($_SERVER['SERVER_PROTOCOL'].' 500 Internal Server Error', true, 500); break;
					case '404': header($_SERVER['SERVER_PROTOCOL'].' 404 Not Found', true, 404); break;
					case '200': header($_SERVER['SERVER_PROTOCOL'].' 200 OK', true, 200); break;
				}
				
				echo $this->config['stub_html_code'];
				exit;
			}
			
		} elseif ($this->config['user_direct_access_action'] == self::USER_ACCESS_ACTION_USER_DOMAIN) {
			if (!$this->config['user_domain'])
				trigger_error('Error in line: '.__LINE__, E_USER_ERROR);
			
			$this->redirectUserToUserdomain();
		}
	}
	
	public function redirectUserToUserdomain() {
		if ($this->config['cookie_that_open_access_on']) {
			$name = $this->config['cookie_that_open_access_name'];
			$expired = 60 * 60 * $this->config['cookie_that_open_access_expired_hours'];
			setcookie($name, 1, time() + $expired, '/');
		}
		
		$url = $this->helper->getSiteProtocol().'://'.$this->config['user_domain'].'/'.ltrim(safe_array_access($_SERVER, 'REQUEST_URI'), '/');
		header('Location: '.$url, true, 302);
		exit;
	}
		
	public function existsOpenSiteReference() {
		if ($this->config['sites_that_open_access_on']) {
			if ($ref = safe_array_access($_SERVER, 'HTTP_REFERER')) {
				$sites = explode("\n", $this->config['sites_that_open_access']);
				$sites = array_map('trim', $sites);
				$sites = array_filter($sites, 'strlen');
				
				foreach ($sites as $site) {
					//$site = parse_url($site, PHP_URL_HOST);
					
					if (strpos($ref, $site) !== false) {
						return true;
					}
				}
			}
		}
		
		return false;
	}
	
	public function existsOpenSiteGetParams() {
		if ($this->config['get_params_that_open_access_on']) {
			if ($_GET) {
				$params = explode("\n", $this->config['get_params_that_open_access']);
				$params = array_map('trim', $params);
				$params = array_filter($params, 'strlen');
				
				foreach ($params as $param) {
					if (isset($_GET[$param])) {
						return true;
					}
				}
			}
		}
		
		return false;
	}
	
	public function eventSearchengineAccessAction() {
		if ($this->config['searchengine_access_action'] == self::SEARCHENGINE_ACCESS_ACTION_BASE_DOMAIN) {
			header('Location: '.$this->getBaseUrl().ltrim(safe_array_access($_SERVER, 'REQUEST_URI'), '/'), true, 301);
			exit;
			
		} elseif ($this->config['searchengine_access_action'] == self::SEARCHENGINE_ACCESS_ACTION_BLANK_PAGE) {
			exit;
		} elseif ($this->config['searchengine_access_action'] == self::SEARCHENGINE_ACCESS_ACTION_NOINDEX) {
			$this->disable_index = true;
		}
	}
	
	public function getBaseUrl() {
		if ($this->config['base_domain']) {
			$url = $this->helper->getSiteProtocol().'://'.$this->config['base_domain'].'/';
		} else {
			$url = $this->site_url;
		}
		
		return $url;
	}
	
	public function getRowById($table, $id, $select_fields = '*') {
		return $this->db->selectRow("SELECT {$select_fields} FROM {$table} WHERE id = ".(int)$id." LIMIT 1");
	}
	
	public function buildChangeTabTagPart($tab, $end_params = '') {
		return 'href="'.$this->module_admin_url.'&tab='.$tab.$end_params.'"';
	}
	
	public function frontendIsEnable() {
		if ($this->config['state']) {
			if (($this->config['enable_for_all'] or $this->user_group == self::ADMIN_USERGROUP_ID) and (!$this->config['test_ip'] or $this->config['test_ip'] == $this->user_ip)) {
				return true;
			}
		}
		
		return false;
	}
	
	public function getCurrTimestamp() {
		return strtotime($this->getCurrDate());
	}
	
	public function getCurrDate() {
		$this->current_date or $this->current_date = $this->db->selectCell("SELECT NOW()");
		return $this->current_date;
	}
	
	public function assignConfig() {
		require ENGINE_DIR.self::CFG_FILEPATH;

		$this->config = $traffcontrol_config;
	}
	
	public function siteMsgError($text, $title = 'Ошибка') {
		msgbox(__($title), __($text));
	}
	
	public function getOldVersionUserHash() {
		if (!$user_id = SESSION('dle_user_id')) {
			$user_id = COOKIE('dle_user_id');
		}
		
		if (!$str = SESSION('dle_password')) {
			$str = COOKIE('dle_password');
		}
		
		if ($user_id and $str) {
			return md5($_SERVER['HTTP_HOST'].$user_id.sha1($str).safe_array_access($this->dle->config, 'key').date('Ymd'));
		}
	}
	
}