<?php

// Checks whether a given array is a list (PHP 8 >= 8.1.0)
if(!function_exists('array_is_list')) {
    function array_is_list($arr) {
        if($arr === []) {
            return true;
        }
        return array_keys($arr) === range(0, count($arr) - 1);
    }
}

// HTTP(s) query
/*
    $url    (string) - Address URL
    $params (array):
        method     (string)       - GET/POST/HEAD
        data       (array/string) - POST data
        cookies    (array/string) - Cookies string
        headers    (array/string) - Additional headers
        user_agent (string)       - User-Agent string
        referrer   (string)       - Referrer URL
        cache      (string)       - Path to the cache directory
        proxy      (array/string) - Use proxy server:
            type     (string)  - Type of proxy server
            server   (string)  - Address of proxy server
            port     (integer) - Port of proxy server
            user     (string)  - Proxy server username
            password (string)  - Proxy server password
        
*/
function http_query($url, $params=[]) {
    $result = null;
    // Get data from cache
    $cache_name = null;
    if(isset($params['cache'])) {
        $parameters = array();
        $reflection = new ReflectionFunction(__FUNCTION__);
        foreach($reflection->getParameters() as $parameter) {
            $name = $parameter->getName();
            $parameters[$name] = ${$name};
        }
		unset($parameters['params']['cache']);
        $cache_file = $params['cache'].'/'.md5(serialize($parameters)).'.cache';
        if(($result = @file_get_contents($cache_file)) !== false) {
            return $result;
        }
    }
    // HTTP method
    if(!isset($params['method'])) {
        $params['method'] = 'GET';
    } else {
        $params['method'] = strtoupper($params['method']);
    }
    // Cookies
    if(isset($params['cookies'])) {
        if(is_array($params['cookies'])) {
            if(array_is_list($params['cookies'])) {
                $params['cookies'] = implode('; ', $params['cookies']);
            } else {
                $params['cookies'] = http_build_query($params['cookies'], '', '; ');
            }
        }
    }
    // Headers
    if(isset($params['headers'])) {
        if(is_array($params['headers'])) {
            if(!array_is_list($params['headers'])) {
                $headers = array();
                foreach($params['headers'] as $key => $value) {
                    $headers[] = "$key: $value";
                }
                $params['headers'] = $headers;
                unset($headers);
            }
        } else {
            $params['headers'] = explode(PHP_EOL, $params['headers']);
        }
    }
    // User agent
    if(!isset($params['user_agent'])) {
        $params['user_agent'] = "$_SERVER[SERVER_NAME]/$_SERVER[SERVER_PORT] ($_SERVER[SERVER_SOFTWARE])";
    }
    // Proxy server
    if(isset($params['proxy'])) {
        if(is_string($params['proxy'])) {
            $proxy_string = $params['proxy'];
            $params['proxy'] = array();
            // proxy type
            $proxy = explode('://', $proxy_string);
            if(count($proxy) > 1) {
                switch(strtoupper($proxy[0])) {
                    case 'SOCKS4' : $params['proxy']['type'] = CURLPROXY_SOCKS4;          break;
                    case 'SOCKS4A': $params['proxy']['type'] = CURLPROXY_SOCKS4A;         break;
                    case 'SOCKS5' : $params['proxy']['type'] = CURLPROXY_SOCKS5;          break;
                    case 'SOCKS5H': $params['proxy']['type'] = CURLPROXY_SOCKS5_HOSTNAME; break;
                    case 'HTTP10' : $params['proxy']['type'] = CURLPROXY_HTTP_1_0;        break;
                    case 'HTTPS'  : $params['proxy']['type'] = CURLPROXY_HTTPS;           break;
                    default: $params['proxy']['type'] = CURLPROXY_HTTP;
                }
                $proxy_string = $proxy[1];
            }
            // user and password
            $proxy_string = $proxy[1];
            $proxy = explode('@', $proxy_string);
            if(count($proxy) > 1) {
                $proxy = explode(':', $proxy[0]);
                if(isset($proxy[0])) { $params['proxy']['user'] = $proxy[0]; }
                if(isset($proxy[1])) { $params['proxy']['password'] = $proxy[1]; }
            }
            // server and port
            $proxy = explode(':', $proxy_string);
            if(isset($proxy[0])) { $params['proxy']['server'] = $proxy[0]; }
            if(isset($proxy[1])) { $params['proxy']['port'] = $proxy[1]; }
        }
    }
    // HTTP query
    if(function_exists('curl_init')) { // cURL
        if($curl = curl_init($url)) {
            curl_setopt($curl, CURLOPT_HEADER, false);
            switch($params['method']) {
                case 'HEAD':
                    curl_setopt($curl, CURLOPT_NOBODY, true);
                    curl_setopt($curl, CURLOPT_HEADER, true);
                    break;
				case 'HEAD+GET':
                case 'GET+HEAD':
					curl_setopt($curl, CURLOPT_HEADER, true);
                case 'GET':
                    curl_setopt($curl, CURLOPT_HTTPGET, true);
                    break;
                case 'HEAD+POST':
				case 'POST+HEAD':
					curl_setopt($curl, CURLOPT_HEADER, true);
                case 'POST':
                    curl_setopt($curl, CURLOPT_POST, true);
                    curl_setopt($curl, CURLOPT_POSTFIELDS, (isset($params['data']) ? $params['data'] : ''));
                    break;
                default:
                    curl_close($curl);
                    return null;
            }
            if(isset($params['referrer'])) { curl_setopt($curl, CURLOPT_REFERER, $params['referrer']); }
            if(isset($params['cookies'])) { curl_setopt($curl, CURLOPT_COOKIE, $params['cookies']); }
            if(!empty($params['user_agent'])) { curl_setopt($curl, CURLOPT_USERAGENT, $params['user_agent']); }
            if(isset($params['headers'])) { curl_setopt($curl, CURLOPT_HTTPHEADER, $params['headers']); }
            if(isset($params['proxy']) && isset($params['proxy']['type'])) { curl_setopt($curl, CURLOPT_PROXYTYPE, $params['proxy']['type']); }
            if(isset($params['proxy']) && isset($params['proxy']['server'])) { curl_setopt($curl, CURLOPT_PROXY, $params['proxy']['server']); }
            if(isset($params['proxy']) && isset($params['proxy']['port'])) { curl_setopt($curl, CURLOPT_PROXYPORT, $params['proxy']['port']); }
            if(isset($params['proxy']) && isset($params['proxy']['user']) && isset($params['proxy']['password'])) {
                curl_setopt($curl, CURLOPT_PROXYUSERPWD, "{$params['proxy']['user']}:{$params['proxy']['password']}");
            }
            curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
            curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, 0);
            curl_setopt($curl, CURLOPT_FOLLOWLOCATION, true);
            curl_setopt($curl, CURLOPT_RETURNTRANSFER, true);
            $result = curl_exec($curl);
            curl_close($curl);
        }
    } else { // file_get_contents
        $headers = array();
        if($params['method'] == 'POST') { $headers[] = 'Content-Type: application/x-www-form-urlencoded'; }
        if(isset($params['referrer'])) { $headers[] = 'Referer: '.$params['referrer']; }
        if(isset($params['cookies'])) { $headers[] = 'Cookie: '.$params['cookies']; }
        if(isset($params['user_agent'])) { $headers[] = 'User-Agent: '.$params['user_agent']; }
        if(isset($params['headers'])) { $headers = array_merge($headers, $params['headers']); }
        // Context options
        $options = array(
            // https://www.php.net/manual/en/context.php
            'http' => [
                'method'           => preg_replace('#(^HEAD\+)|(\+HEAD$)#s', '', $params['method']),
                'header'           => implode(PHP_EOL, $headers),
                'content'          => http_build_query(isset($params['data']) ?  $params['data'] : []),
                'ignore_errors'    => true
            ],
            'ssl' => [
                'verify_peer'      => false,
                'verify_peer_name' => false,
            ]
        );
        // Proxy
        if(isset($params['proxy']) && isset($params['proxy']['server'])) {
            // https://stackoverflow.com/questions/1336262/file-get-contents-behind-a-proxy
            $options['http']['request_fulluri'] = true;
            $options['http']['proxy'] = "tcp://{$params['proxy']['server']}";
            if(isset($params['proxy']) && isset($params['proxy']['port'])) {
                $options['http']['proxy'] .= ":{$params['proxy']['port']}";
            }
            if(isset($params['proxy']) && isset($params['proxy']['user']) && isset($params['proxy']['password'])) {
                $options['http']['header'] .= PHP_EOL.'Proxy-Authorization: Basic '.base64_encode("{$params['proxy']['user']}:{$params['proxy']['password']}");
            }
        }
        // Do it
        $result = @file_get_contents($url, false, stream_context_create($options));
        if((strpos($params['method'], 'HEAD') !== false) && ($result !== false)) {
			if($params['method'] == 'HEAD') {
				$result = implode(PHP_EOL, $http_response_header);
			} else {
				$result = implode(PHP_EOL, $http_response_header).PHP_EOL.PHP_EOL.$result;
			}
        }
    }
    // Write to cache
    if(!is_null($result) && isset($cache_file)) {
        @mkdir(dirname($cache_file), 0755, true);
        @file_put_contents($cache_file, $result);
    }
    return $result;
}

// Get text from tags
function get_from_tags($opening_tag, $closing_tag, $html) {
    $result = array();
    $opening_tag = str_replace('#', '\#', $opening_tag);
    $closing_tag = str_replace('#', '\#', $closing_tag);
    if(preg_match_all('#'.$opening_tag.'(.*)'.$closing_tag.'#isU', $html, $matches)) {
        $result = $matches[1];
    }
    return $result;
}

?>
